/**
 * @author Joel Espinosa Longi
 * @licencia LGPL - http://www.gnu.org/licenses/lgpl.html
 */

var diaEd = (function(diaEd) {

  /**
   * 
   */
  diaEd.Attributes = function(content, type) {
    this.attributes = {};

    // the attributes belong to a dialog
    if (type === "dialog") {
      this.attributes = {
        title         : "",
        spellcheck    : "auto",
        autoresize    : false,
        delay         : 50,
        withAudio     : false,
        tts           : false,
        withoutServer : false,
        styleBalloons : "css",
        firstQuestion : "q1"
      }
    }
    // the attributes belong to a question
    else if (type === "question") {
      this.attributes = {
        id                  : "q1",
        x                   : 25,
        y                   : 25,
        text                : "",
        audio               : "",
        display             : "",
        hide                : "",
        set                 : "",
        nrRequiredResponses : 1,
        prompt              : "",
        nextQuestion        : "",
        endDialogue         : false,
        collapsed           : false
      }
    }
    // the attributes belong to an answer
    else if (type === "answer") {
      this.attributes = {
        shortKey     : "",
        RADialKey    : "",
        DescartesKey : "",
        regexKey     : "",
        bestAnswer   : "",
        feedback     : "",
        audio        : "",
        display      : "",
        hide         : "",
        set          : "",
        nextQuestion : "",
        weight       : 1,
        endDialogue  : false,
        description  : "",
      }
    }

    else if (type === "microworld") {
      this.attributes = {
        id                    : "mw1",
        url                   : "",
        w                     : null,
        h                     : null,
        x                     : null,
        y                     : null,
        front                 : false,
        autoDisplay           : false,
        textLink              : "",
        deactivateChat        : false,
        toNextQuestionOnClose : false,
        reset                 : false
      }
    }

    // traverse the values to replace the defaults values of the object
    for (var propName in content) {
      // verify the own properties of the object
      if (content.hasOwnProperty(propName)) {
        this.attributes[propName] = content[propName];
      }
    }

    // remove extra attributes
    delete this.attributes["questions"];
    delete this.attributes["answers"];
    delete this.attributes["microWorlds"];
  }

  /**
   *
   */
  diaEd.Attributes.prototype.get = function(name) {
    return this.attributes[name];
  }

  /**
   *
   */
  diaEd.Attributes.prototype.set = function(name, value) {
    this.attributes[name] = value;
  }

  /**
   *
   */
  diaEd.Attributes.prototype.toArray = function() {
    var array = [];
    for ( var attr in this.attributes ) {
      if (this.attributes.hasOwnProperty(attr)) {
        array.push({ name: attr, value: this.attributes[attr] });
      }
    }
    return array;
  }

  /**
   *
   */
  diaEd.Attributes.prototype.toString = function(nivel) {
    var tabularNumber = new Array(++nivel).join(diaEd.tabularString);

    var str = "";
    for ( var attr in this.attributes ) {
      if (this.attributes.hasOwnProperty(attr)) {
        str += tabularNumber + '"' + attr + '" : ' + JSON.stringify(this.get(attr)) + ',\n';
      }
    }

    return str.substring(0,str.length-2);
  }

  return diaEd;
})(diaEd || {});