/**
 * @author Joel Espinosa Longi
 * @licencia LGPL - http://www.gnu.org/licenses/lgpl.html
 */

var diaEd = (function(diaEd) {

  // used for the string output of the dialog
  diaEd.tabularString = "  ";
  diaEd.filename = "dialogo.json";
  diaEd.dialog = null;
  diaEd.elementToDelete = null;

  /**
   * Init the editor
   */
  $(document).ready(function() {
    
    ////////////////////////////////////////////
    // nodewebkit code
    if (window.require) {
      new diaEd.GUI();
    }
    ////////////////////////////////////////////

    // create the message windows
    $("#message_deleteAnswer").dialog({ resizable: false, autoOpen: false, draggable: false, height: 160, modal: true,
                                        buttons: {
                                          "Borrar": function() {
                                            $(diaEd.elementToDelete.self).parent().remove();
                                            diaEd.elementToDelete.question.cleanAnswerList();
                                            diaEd.elementToDelete.question.render();

                                            diaEd.attributesPanel.show("#dialogAttributes");

                                            diaEd.dialog.init();

                                            $(this).dialog( "close" );
                                          },
                                          Cancelar: function() {
                                            $(this).dialog( "close" );
                                          }
                                        }
    });

    $("#message_deleteQuestion").dialog({ resizable: false, autoOpen: false, draggable: false, height: 160, modal: true,
                                          buttons: {
                                            "Borrar": function() {
                                              $(diaEd.elementToDelete.self).parent().remove();
                                              diaEd.dialog.cleanQuestionList();

                                              diaEd.attributesPanel.show("#dialogAttributes");

                                              diaEd.dialog.init();

                                              $(this).dialog( "close" );
                                            },
                                            Cancelar: function() {
                                              $(this).dialog( "close" );
                                            }
                                          }
    });

    $("#message_deleteMicroworld").dialog({ resizable: false, autoOpen: false, draggable: false, height: 160, modal: true,
                                            buttons: {
                                              "Borrar": function() {
                                                $(diaEd.elementToDelete.self).parent().remove();
                                                diaEd.dialog.cleanMicromundoList();

                                                diaEd.attributesPanel.show("#dialogAttributes");

                                                $(this).dialog( "close" );
                                              },
                                              Cancelar: function() {
                                                $(this).dialog( "close" );
                                              }
                                            }
    });
    

    ////////////////////////////////////////////
    // create spinners
    $("#dialogAttributes_delay").spinner( { min: 0 } );
    
    $("#questionAttributes_nrRequiredResponses").spinner( { min: 1 } );
    
    $("#answerAttributes_weight").spinner( { min: 0 } );

    $("#micromundoAttributes_w").spinner( { min: 0 } );
    $("#micromundoAttributes_h").spinner( { min: 0 } );
    $("#micromundoAttributes_x").spinner();
    $("#micromundoAttributes_y").spinner();

    ////////////////////////////////////////////
    // configure tabs
    $("#tabs").tabs();

    ////////////////////////////////////////////
    // add an event listener to the hidden elements, needed to open and save files
    $("#files").change(diaEd.onHandleFileSelect);
    $("#save").click(diaEd.onSave);

    ////////////////////////////////////////////
    // the text editor
    CKEDITOR.replace("realTextEditor");

    // the accept button for the text editor panel
    $("#accept_btn").button();
    $("#accept_btn").click(function(evt) {
      if (diaEd.currentField) {
        $(diaEd.currentField).val( CKEDITOR.instances.realTextEditor.getData() ); 
        $(diaEd.currentField).trigger("change");
      }

      $("#textEditor").hide();
    });
    // the cancel button for the text editor panel
    $("#cancel_btn").button();
    $("#cancel_btn").click(function(evt) {
      $("#textEditor").hide();
    });

    ////////////////////////////////////////////
    // register the text fields that open the text editor
    function mousedownText(evt) {
      CKEDITOR.instances.realTextEditor.setData( $(this).val() );

      diaEd.currentField = this;

      $("#textEditor").show();
    }
    $("#questionAttributes_text").mousedown(mousedownText);
    $("#answerAttributes_feedback").mousedown(mousedownText);

    ////////////////////////////////////////////
    // register the text fields that open the RADial expression editor
    function mousedownRadial(evt) {
      // $("#radialExprEditorIframe").get(0).src = "http://arquimedes.matem.unam.mx/DescartesHTML5/pruebasJ/editorExprRADial_v2/editor/editorClavesRADial.html?shortKey="+$("#answerAttributes_shortKey").val()+"&RADialKey="+$("#answerAttributes_RADialKey").val()+"&spellcheck=auto&withoutServer=true";

      $("#radialExprEditorIframe").get(0).contentWindow.postMessage({ type: "setKeys", 
                                                                      shortKey: $("#answerAttributes_shortKey").val(),  
                                                                      radialKey: $("#answerAttributes_RADialKey").val()
                                                                    }, "*");

      $("#radialExprEditor").show();
    }
    $("#answerAttributes_shortKey").mousedown(mousedownRadial);
    $("#answerAttributes_RADialKey").mousedown(mousedownRadial);

    // register a message manager to get the values from the RADial expression editor
    window.addEventListener("message", function(evt) {
      var data = evt.data;
      if (data.type === "setKeys") {
        if (data.shortKey == "(___)") {
          data.shortKey = "";
        }
        if (data.radialKey == "(___)") {
          data.radialKey = "";
        }

        $("#answerAttributes_shortKey").val(data.shortKey);
        $("#answerAttributes_RADialKey").val(data.radialKey);

        diaEd.attributesPanel.save(diaEd.currentPanel);

        $("#radialExprEditor").hide();
      }
    });

    ////////////////////////////////////////////
    function clickCommondBodyAndTabs() {
      // unmark the question
      if ((diaEd.currentQuestion != null) && (diaEd.currentQuestion != self)) {
        $(diaEd.currentQuestion.container).removeClass("selectedElement");
      }
      // unmark the answer
      if ((diaEd.currentAnswer != null) && (diaEd.currentAnswer != self)) {
        $(diaEd.currentAnswer.container).removeClass("selectedAnswer");
      }
      // unmark the microworld
      if ((diaEd.currentMicromundo != null) && (diaEd.currentMicromundo != self)) {
        $(diaEd.currentMicromundo.container).removeClass("selectedElement");
      }

      diaEd.attributesPanel.save(diaEd.currentPanel);
      diaEd.attributesPanel.show("#dialogAttributes");
    }

    // add the event listener to the body
    $("body").mousedown(function(evt) {
      // a dialog is loaded
      if (diaEd.dialog != undefined) {
        // click in contentEditor or micromundoEditor
        if ( (evt.target === $("#contentEditor").get(0)) ||
             (evt.target === $("#micromundoEditor").get(0))
           ) {

          clickCommondBodyAndTabs();
        }
      }
    });

    // click tabs
    function clickTab(evt) {
      if (diaEd.dialog != undefined) {
        clickCommondBodyAndTabs();
      }
    }
    $("#tab1").click(clickTab);
    $("#tab2").click(clickTab);

    ////////////////////////////////////////////
    // add the event listener to the attributes container, for save the change values
    function changeFun(evt) {
      // change in contentEditor
      if ($("#dialogAttributes").css("display") !== "none") {
        diaEd.attributesPanel.save("#dialogAttributes");
      }

      // change in question
      if ($("#questionAttributes").css("display") !== "none") {
        diaEd.attributesPanel.save("#questionAttributes");
      }

      // change in question
      if ($("#answerAttributes").css("display") !== "none") {
        diaEd.attributesPanel.save("#answerAttributes");
      }

      // change in microworld
      if ($("#micromundoAttributes").css("display") !== "none") {
        diaEd.attributesPanel.save("#micromundoAttributes");
      }

      // change the arrows if needed
      // if ( (evt.target == $("#answerAttributes_nextQuestion").get(0)) || (evt.target == $("#questionAttributes_id").get(0)) ) {
      //   diaEd.dialog.init();
      //   diaEd.lineDrawing.update();
      // }

      if (evt.target == $("#answerAttributes_nextQuestion").get(0)) {
        diaEd.dialog.init(true);
        diaEd.lineDrawing.update();
      }
      if (evt.target == $("#questionAttributes_id").get(0)) {
        diaEd.dialog.init();
        diaEd.lineDrawing.update();
      }

      // conditions to enable or disable attributes
      diaEd.enableAudio = diaEd.dialog.attributes.get("withAudio") && !diaEd.dialog.attributes.get("tts");
    }

    $("#attributes").change(changeFun);
    $("#attributes").on( "spin", changeFun);

    ////////////////////////////////////////////
    diaEd.lineDrawing = new diaEd.LineDrawing( $("#svgContainer") );

    ////////////////////////////////////////////
    // configure the toolbar
    diaEd.configureToolbar();

    $("#add_question_btn").mousedown(function(evt) {
      if (diaEd.dialog) {
        diaEd.dialog.addQuestion();
      }
    });

    $("#add_microworld_btn").mousedown(function(evt) {
      if (diaEd.dialog) {
        diaEd.dialog.addMicromundo();
      }
    });

    ////////////////////////////////////////////
    // controls verifications for enable or disable elements
    $("#dialogAttributes_withAudio").change( function(evt) {
      var checked = $(this).prop("checked");

      if (checked) {
        $("#dialogAttributes_tts").removeAttr("disabled");
      }
      else {
        $("#dialogAttributes_tts").attr("disabled", "disabled");
        $("#dialogAttributes_tts").prop("checked", false);
      }
    });

    $("#questionAttributes_nrRequiredResponses").on( "spinstop", function(evt) {
      var isDefaultValue = $(this).val() == 1;

      if (isDefaultValue) {
        $("#questionAttributes_prompt").removeAttr("disabled");
        $("#questionAttributes_nextQuestion").removeAttr("disabled");
      }
      else {
        $("#questionAttributes_prompt").val("");
        $("#questionAttributes_prompt").attr("disabled", "disabled");
        $("#questionAttributes_nextQuestion").val("");
        $("#questionAttributes_nextQuestion").attr("disabled", "disabled");
      }
    });

    $("#micromundoAttributes_front").change( function(evt) {
      var checked = $(this).prop("checked");

      if (checked) {
        $("#micromundoAttributes_autoDisplay").removeAttr("disabled");
        $("#micromundoAttributes_toNextQuestionOnClose").removeAttr("disabled");
      }
      else {
        $("#micromundoAttributes_autoDisplay").attr("disabled", "disabled");
        $("#micromundoAttributes_autoDisplay").prop("checked", false);
        $("#micromundoAttributes_toNextQuestionOnClose").attr("disabled", "disabled");
        $("#micromundoAttributes_toNextQuestionOnClose").prop("checked", false);        
      }

    });

  });

  return diaEd;
})(diaEd || {});