/**
 * @author Joel Espinosa Longi
 * @licencia LGPL - http://www.gnu.org/licenses/lgpl.html
 */

var diaEd = (function(diaEd) {

  /**
   * 
   */
  diaEd.Question = function(content, parent) {
    this.parent = parent;

    this.attributes = new diaEd.Attributes(content, "question");
    if (content) {
      this.answers = this.getAnswers(content.answers);
    } else {
      this.answers = this.getAnswers();
    }
  }

  /**
   *
   */
  diaEd.Question.prototype.getAnswers = function(childNodes) {
    childNodes = childNodes || [];
    var answers = [];

    for (var i=0, l=childNodes.length; i<l; i++) {
      answers.push( new diaEd.Answer(childNodes[i], this) );
    }

    if (answers.length === 0) {
      answers.push( new diaEd.Answer([], this) );
    }

    return answers;
  }

  /**
   *
   */
  diaEd.Question.prototype.addNewAnswer = function() {
    this.cleanAnswerList();

    this.answers.push( new diaEd.Answer([], this) );
    this.render(this.parent);
  }

  /**
   *
   */
  diaEd.Question.prototype.cleanAnswerList = function() {
    var children = $(this.childrenList).children();
    var tmpAns = [];

    for (var i=0, l=children.length; i<l; i++) {
      for (var j=0, k=this.answers.length; j<k; j++) {
        if ($(this.answers[j].container)[0] === children[i]) {
          tmpAns.push(this.answers[j]);
        }
      }      
    }    

    this.answers = tmpAns;
  }

  /**
   *
   */
  diaEd.Question.prototype.buildArrows = function() {
    for (var i=0,l=this.answers.length; i<l; i++) {
      this.answers[i].buildArrowNextQuestion();
    }
  }

  /**
   *
   */
  diaEd.Question.prototype.render = function(parent) {
    var self = this;
    parent = parent || self.parent;

    if (self.container) {
      self.container.remove();
    }

    this.container = $("<div></div>");

    parent.append(this.container);

    this.container.addClass("questionContainer");

    this.container.draggable({ 
      containment: "parent", 
      cursor:      "crosshair", 
      grid:        [ 5, 5 ], 
      opacity:     0.45, 
      scroll:      false,
      drag: function(evt, ui) {
        diaEd.lineDrawing.update();
      },
      stop: function(evt, ui) {
        diaEd.lineDrawing.update();
      }
    });

    // positon and style of the question container
    this.container.css( {
      left:     this.attributes.get("x"), 
      top:      this.attributes.get("y"),
      position: "absolute"
    });

    /***********************************************************************************************/
    this.divSpanText = $("<div style='padding-left: 7px;'></div>")
    this.spanText = $("<div class='spanText' style='width: 170px; '>" + this.attributes.get("id") + "</div>")
    this.divSpanText.append(this.spanText);
    this.container.append(this.divSpanText);

    this.accordionContainer = $("<div style='padding-left: 18px; padding-top: 0px; overflow: visible; background: #ffb;'></div>");
    this.container.append(this.accordionContainer);

    this.accordionContainer.mouseover(function(evt) {
      evt.stopPropagation();
    });

    this.container.accordion({ 
      collapsible: true,
      animate: false,
      heightStyle: "content",

      event: false,

      activate: function(evt, ui) {
        diaEd.lineDrawing.update();
      }
    });

    var isCollapsed = this.attributes.get("collapsed");
    if (isCollapsed) {
      $(this.container).accordion( "option", "active", false );
    }

    /***********************************************************************************************/
    this.uncollapseBtn = $("<div></div>");
    $(this.uncollapseBtn).button( { icons: { primary: "my-icon-uncollapse"  }, text: false } ).addClass("collapseButton");
    this.container.append(this.uncollapseBtn);

    this.collapseBtn = $("<div></div>");
    $(this.collapseBtn).button( { icons: { primary: "my-icon-collapse"  }, text: false } ).addClass("collapseButton");
    this.container.append(this.collapseBtn);

    if (isCollapsed) {
      this.collapseBtn.hide();
    }

    this.collapseBtn.click(function(evt) {
      $(self.container).accordion( "option", "active", false );
      self.collapseBtn.hide();
    });
    this.uncollapseBtn.click(function(evt) {
      $(self.container).accordion( "option", "active", 0 );
      self.collapseBtn.show();
    });

    /***********************************************************************************************/
    this.removeBtn = $("<div></div>");
    $(this.removeBtn).button( {icons: { primary: "my-icon-delete" }, text: false } ).addClass("collapseButton");
    this.removeBtn.css( { left: "202px", top:  "5px" });
    /**
     *
     */
    this.removeBtn.mousedown(function(evt) {
      evt.stopPropagation();

      diaEd.elementToDelete = { self: this };

      $("#message_deleteQuestion").dialog("open");
    })
    this.container.append(this.removeBtn);

    /***********************************************************************************************/
    var tmpX, tmpY;
    /**
     *
     */
    this.container.mousedown(function(evt) {
      tmpX = evt.clientX;
      tmpY = evt.clientY;

      $(self.container).addClass("selectedElement");

      if (evt.target.tagName.toLowerCase() !== "li") {
        if (evt.target == self.accordionContainer.get(0)) {
          // put the current dragged element in the top      
          $(this).parent().append(this);
        }

        // unmark the question
        if ((diaEd.currentQuestion != null) && (diaEd.currentQuestion != self)) {
          $(diaEd.currentQuestion.container).removeClass("selectedElement");
        }
        // unmark the answer
        if ((diaEd.currentAnswer != null) && (diaEd.currentAnswer != self)) {
          $(diaEd.currentAnswer.container).removeClass("selectedAnswer");
        }

        diaEd.attributesPanel.save(diaEd.currentPanel);
        diaEd.currentQuestion = self;
        diaEd.attributesPanel.show("#questionAttributes");
      }
    });

    /**
     *
     */
    this.container.mouseup(function(evt) {
      if (evt.target.tagName.toLowerCase() !== "li") {
          self.attributes.set("x", parseInt( $("#divContainer").scrollLeft() ) + parseInt( self.container.position().left ));
          self.attributes.set("y", parseInt( $("#divContainer").scrollTop()  ) + parseInt( self.container.position().top ));
      }
    });    

    /***********************************************************************************************/
    // children list
    this.childrenList = $("<ul class='answerList'></ul>");
    this.childrenList.sortable({
      containment: "parent",
      
      sort: function(evt, ui) {
        diaEd.lineDrawing.update();
      },
      stop: function(evt, ui) {
        diaEd.lineDrawing.update();
      }
    });
    this.childrenList.disableSelection();

    for (var i=0, l=this.answers.length; i<l; i++) {
      this.answers[i].render(this.childrenList);
    }

    /**
     *
     */
    this.childrenList.on("sortstop", function(evt, ui) {
      self.cleanAnswerList();
    });

    this.accordionContainer.append(this.childrenList);

    /***********************************************************************************************/
    this.addAnswer = $("<div class='addAnswer'>+ Agregar respuesta</div>");
    this.addAnswer.addClass("ui-widget-content");

    this.addAnswer.disableSelection();

    this.addAnswer.mousedown(function(evt) {
      diaEd.attributesPanel.save(diaEd.currentPanel);

      diaEd.currentQuestion = self;
      diaEd.attributesPanel.show("#questionAttributes");

      self.addNewAnswer();
      $(self.container).addClass("selectedElement");

      // unmark the question
      if ((diaEd.currentQuestion != null) && (diaEd.currentQuestion != self)) {
        $(diaEd.currentQuestion.container).removeClass("selectedElement");
      }

      diaEd.currentQuestion = self;
    });
    this.accordionContainer.append(this.addAnswer);

  }

  /**
   * Fill the properties panel with the attributes of the question
   */
  diaEd.Question.prototype.fillPanel = function(panelName) {
    // get identifier
    $(panelName + "_id").val( this.attributes.get("id"));

    // get text
    $(panelName + "_text").val( this.attributes.get("text"));

    // get audio
    if (diaEd.enableAudio) {
      $(panelName + "_audio").removeAttr('disabled');
      $(panelName + "_audio").val( this.attributes.get("audio"));
    }
    else {
      $(panelName + "_audio").attr("disabled", "disabled");
      $(panelName + "_audio").val("");
    }

    // get display
    $(panelName + "_display").val( this.attributes.get("display"));

    // get hide
    $(panelName + "_hide").val( this.attributes.get("hide"));

    // get set
    $(panelName + "_set").val( this.attributes.get("set"));

    // get nrRequiredResponses
    var withnrRequiredResponsesDefault = (this.attributes.get("nrRequiredResponses") == 1);
    $(panelName + "_nrRequiredResponses").val( this.attributes.get("nrRequiredResponses"));

    if (withnrRequiredResponsesDefault) {
      // get prompt
      $(panelName + "_prompt").val( this.attributes.get("prompt"));

      // get nextQuestion
      $(panelName + "_nextQuestion").val( this.attributes.get("nextQuestion"));
    }
    else {
      $(panelName + "_prompt").val("");
      $(panelName + "_prompt").attr("disabled", "disabled");

      $(panelName + "_nextQuestion").val("");
      $(panelName + "_nextQuestion").attr("disabled", "disabled");
    }

    // get endDialogue
    $(panelName + "_endDialogue").prop("checked", this.attributes.get("endDialogue"));
  }

  /**
   *
   */
  diaEd.Question.prototype.savePanelContent = function(panelName) {
    // set identifier
    this.attributes.set("id", $(panelName + "_id").val());
    this.spanText.text(this.attributes.get("id"));

    // set text
    this.attributes.set("text", $(panelName + "_text").val());

    // set audio
    this.attributes.set("audio", $(panelName + "_audio").val());

    // set display
    this.attributes.set("display", $(panelName + "_display").val());

    // set hide
    this.attributes.set("hide", $(panelName + "_hide").val());

    // set set
    this.attributes.set("set", $(panelName + "_set").val());

    // set nrRequiredResponses
    var tmpval = parseInt( $(panelName + "_nrRequiredResponses").val() );
    this.attributes.set("nrRequiredResponses", isNaN(tmpval)?0:tmpval);

    // set prompt
    this.attributes.set("prompt", $(panelName + "_prompt").val());

    // set nextQuestion
    this.attributes.set("nextQuestion", $(panelName + "_nextQuestion").val());

    // set endDialogue
    this.attributes.set("endDialogue", $(panelName + "_endDialogue").prop("checked") );

    // the question is collapsed
    this.attributes.set("collapsed", ($(this.container).accordion("option", "active") === false) ? true : false );
  }  

  /**
   *
   */
  diaEd.Question.prototype.toString = function(nivel) {
    var tabularNumber = new Array(++nivel).join(diaEd.tabularString);

    var str = new Array(nivel+1).join(diaEd.tabularString) + "{\n" + this.attributes.toString(nivel+1);

    // has some answers
    if (this.answers.length > 0) {
      str += ',\n' + new Array(nivel+2).join(diaEd.tabularString) + '"answers" : [\n';

      for (var i=0, l=this.answers.length; i<l; i++) {
        str += this.answers[i].toString(nivel+1) + ",\n";
      }

      str = str.substring(0, str.length-2);
      str += "\n" + new Array(nivel+2).join(diaEd.tabularString) + "]"
    }

    str += "\n" + new Array(nivel+1).join(diaEd.tabularString) + "},";

    return str;
  }

  return diaEd;
})(diaEd || {});